--[[
	Description: Titan Panel plugin that shows your Profession level
	Author: Eliote
	This addon was created and developed after Canettieri's "Titan Panel [Professions] Multi".
--]]

local _, L = ...;
local ACE = LibStub("AceLocale-3.0"):GetLocale("Titan", true)
local VERSION = GetAddOnMetadata("TitanProfession", "Version")

local PROFESSION_LEVEL_LIMIT = 700
local PANDAREM_LIMIT = 600
local NAME_PREFIX = "" --"|c0070DE5f"

local function interp(str, tab)
    return (str:gsub('($%b{})', function(w) return tostring(tab[w:sub(3, -2)] or w) end))
end

local function CanLevelUp(profLvl, profMaxLvl)
    if profMaxLvl == PROFESSION_LEVEL_LIMIT then return end
    if profMaxLvl == PANDAREM_LIMIT then return end -- need test

    if profLvl > (profMaxLvl - 25) then return true end
end

local function GetProfLvlColor(profLvl, profMaxLvl)
    if CanLevelUp(profLvl, profMaxLvl) then
        if profLvl == profMaxLvl then
            return "|cFFFF2e2e"
        else
            return  "|cFFFFFF6a"
        end
    end

    return "|cFFFFFFFF"
end

local function TitanPanelProfButton_OnClick(self, button, offset, castSkill)
    if (button == "LeftButton") then
        if offset > 0 and castSkill then
            CastSpell(offset + castSkill, "Spell")
        end
    end
end

local function GetProfButtonText(profLvl, profMaxLvl, profBonus, profName, defaultDesc)
    local bonusText1 = ""

    if profBonus and profBonus > 0 then
        bonusText1 = "|cFF69FF69+" .. profBonus .. "|r(|cFF69FF69" .. (profLvl + profBonus) .. "|r)"
    end

    --[[if profLvl == PROFESSION_LEVEL_LIMIT then
        return profName .. ": ", "|cFF69FF69" .. bonusText1
    else]]
    if profMaxLvl == 0 then
        return profName .. ": ", "|cFFFF2e2e" .. defaultDesc
    else
        return profName .. ": ", GetProfLvlColor(profLvl, profMaxLvl) .. profLvl .. bonusText1 .. "|r/|cFFFF2e2e" .. profMaxLvl
    end
end

local function PrepareMenu(id)
    TitanPanelRightClickMenu_AddTitle(TitanPlugins[id].menuText)
    TitanPanelRightClickMenu_AddToggleIcon(id)
    TitanPanelRightClickMenu_AddToggleLabelText(id)
    TitanPanelRightClickMenu_AddSpacer()
    TitanPanelRightClickMenu_AddCommand(ACE["TITAN_PANEL_MENU_HIDE"], id, TITAN_PANEL_MENU_FUNC_HIDE);
end

local function TitanProf(idPrefix, profIndex, castSkill, defaultDesc, showDef)
    local ID = idPrefix .. profIndex

    local profName = defaultDesc
    local profIcon = ""
    local profLevel = 0
    local profMaxLevel = 0
    local profBonus = 0
    local profOffset = 0

    local tooltipDescription = interp(L["tooltipDescription"], { prof = profName })

    local elap = 0

    -- Main button frame and addon base
    local frame = CreateFrame("Button", "TitanPanel" .. ID .. "Button", CreateFrame("Frame", nil, UIParent), "TitanPanelComboTemplate")
    frame:SetFrameStrata("FULLSCREEN")
    frame:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)
    frame:RegisterEvent("ADDON_LOADED")
    frame:RegisterEvent("PLAYER_ENTERING_WORLD")
    frame:SetScript("OnClick", function(self, button, ...)
        TitanPanelProfButton_OnClick(self, button, profOffset, castSkill)
        TitanPanelButton_OnClick(self, button)
    end)

    function frame.SetVars(name, icon, level, maxLevel, offset, bonus)
        if profName == name and
                profIcon == icon and
                profLevel == level and
                profMaxLevel == maxLevel and
                profOffset == offset and
                profBonus == bonus
        then
            return
        end

        profOffset = offset or 0
        profIcon = icon or "Interface\\Icons\\INV_Misc_QuestionMark"
        profName = name or defaultDesc
        profLevel = level or 0
        profMaxLevel = maxLevel or 0
        profBonus = bonus or 0

        TitanPlugins[ID].icon = profIcon
        TitanPlugins[ID].tooltipTitle = profName

        tooltipDescription = ""

        if name then
            if showDef then
                TitanPlugins[ID].menuText = NAME_PREFIX .. " " .. defaultDesc .. " [|c0070DE5f" .. profName .. "|r]"
            else
                TitanPlugins[ID].menuText = NAME_PREFIX .. " " .. defaultDesc .. "|r"
            end

            if CanLevelUp(profLevel, profMaxLevel) then
                tooltipDescription = L["goTrainerHint"].."\r"
            end

            if castSkill then
                tooltipDescription = tooltipDescription .. L["tooltipHint"] .. "|r\r\r" .. interp(L["tooltipDescription"], { prof = profName })
            else
                tooltipDescription = tooltipDescription .. "|r\r" .. interp(L["tooltipDescription"], { prof = profName })
            end
        else
            TitanPlugins[ID].menuText = NAME_PREFIX .. " " .. defaultDesc .. " [|cFFFF2e2e" .. L["noprof"] .. "|r]"

            tooltipDescription = L["tooltipNoProfHint"]
        end

        TitanPanelButton_UpdateButton(ID)
    end

    function frame:PLAYER_ENTERING_WORLD()
        frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
        frame.PLAYER_ENTERING_WORLD = nil

        local prof = select(profIndex, GetProfessions())
        if prof then
            local name = GetProfessionInfo(prof)
            frame.SetVars(name)
        else
            frame.SetVars()
        end
    end

    function frame:ADDON_LOADED(a1)
        if a1 ~= "TitanProfession" then
            return
        end

        self:UnregisterEvent("ADDON_LOADED")
        self.ADDON_LOADED = nil

        self.registry = {
            id = ID,
            menuText = NAME_PREFIX .. " " .. defaultDesc .. "|r",
            buttonTextFunction = "TitanPanel" .. ID .. "Button_GetButtonText",
            tooltipTitle = L["noprof"],
            tooltipTextFunction = "TitanPanel" .. ID .. "Button_GetTooltipText",
            frequency = 1,
            icon = "",
            iconWidth = 16,
            category = "Profession",
            version = VERSION,
            savedVariables = {
                ShowIcon = 1,
                ShowLabelText = false,
            },
        }
        self:SetScript("OnUpdate", function(this, a1)
            elap = elap + a1
            if elap < 1 then return end

            local prof = select(profIndex, GetProfessions())

            if not prof then return frame.SetVars() end

            local name, icon, level, maxLevel, _, offset, _, skillModifier = GetProfessionInfo(prof)

            frame.SetVars(name, icon, level, maxLevel, offset, skillModifier)

            elap = 0
        end)
    end

    _G["TitanPanelRightClickMenu_Prepare" .. ID .. "Menu"] = function() PrepareMenu(ID) end
    _G["TitanPanel" .. ID .. "Button_GetButtonText"] = function() return GetProfButtonText(profLevel, profMaxLevel, profBonus, profName, defaultDesc) end
    _G["TitanPanel" .. ID .. "Button_GetTooltipText"] = function() return tooltipDescription end
end

TitanProf("TITAN_PROF_", 1, 1, L["prof1"], true) -- prof1
TitanProf("TITAN_PROF_", 2, 1, L["prof2"], true) -- prof2
TitanProf("TITAN_PROF_", 3, 1, L["archaeology"]) -- archaeology
TitanProf("TITAN_PROF_", 4, nil, L["fishing"])   -- fishing
TitanProf("TITAN_PROF_", 5, 1, L["cooking"])     -- cooking
TitanProf("TITAN_PROF_", 6, 1, L["firstAid"])    -- firstAid

